import java.util.Vector;

/**
 * Representa um conjunto de elementos agrupados em clusters.
 * 
 * @author Joo Carlos Pascoal Faria
 * @version 6 de Fevereiro de 2003
 */
public class ClusteredSet
{
    /**
     * Distancia mxima entre um ponto e o ponto mais prximo do mesmo cluster
     */
    private double maxDistance;
    
    /**
     * Conjunto de clusters representado por um objecto da classe Vector do java.
     */
    private Vector clusters = new Vector();

    /**
     * Constri um conjunto inicialmente vazio.
     * @param maxDistance Distancia mxima entre um ponto e o ponto mais prximo 
     *                    do mesmo cluster
     */
    public ClusteredSet(double maxDistance)
    {
        this.maxDistance = maxDistance;
    }

    /**
     * Adiciona um elemento aos clusters existentes.
     * Pode implicar a criao de um novo cluster, a juno a um cluster
     * j existente ou mesmo a fuso de vrios clusters (no caso do novo elemento
     * poder pertencer a vrios clusters, esses clusters so reunidos num maior).
     * 
     * @param  elem elemento a adicionar
     */
    public void addElement(ClusterElement elem)
    {
        Cluster destinationCluster = null;
        
        for (int i=0; i < clusters.size(); i++)
        {
            Cluster cluster = (Cluster)clusters.elementAt(i);
            if (cluster.mayAddElement(elem))
                if (destinationCluster == null)
                {
                    cluster.addElement(elem);
                    destinationCluster = cluster;
                }
                else
                {
                    destinationCluster.addCluster(cluster);
                    clusters.removeElementAt(i);
                    i--;
                }
         }
         
         if (destinationCluster == null)
         {
            destinationCluster = new Cluster(maxDistance);
            destinationCluster.addElement(elem);
            clusters.addElement(destinationCluster);
         }
    }
    
    /**
     * Repreenta o conjunto de clusters numa string no formato 
     *   {cluster1,
     *    cluster2,
     *    ...
     *    clusterk}
     * 
     * @return a string pretendida
     */
    public String toString()
    {
        String s = "{";
        for (int i=0; i < clusters.size(); i++)
        {
            if (i > 0)
                s = s + ",\n ";
            s = s + clusters.elementAt(i);
        }    
        s = s + "}";
        return s;
     }
}